/*******************************************************************************
* Copyright (c) 2021, Bayerische Motoren Werke Aktiengesellschaft (BMW AG)
*
* This program and the accompanying materials are made
* available under the terms of the Eclipse Public License 2.0
* which is available at https://www.eclipse.org/legal/epl-2.0/
*
* SPDX-License-Identifier: EPL-2.0
*******************************************************************************/

//-----------------------------------------------------------------------------
/** @file  i_environment.h */
//-----------------------------------------------------------------------------


#ifndef MANTLEAPI_EXECUTION_IENVIRONMENT_H
#define MANTLEAPI_EXECUTION_IENVIRONMENT_H

#include <MantleAPI/EnvironmentalConditions/road_condition.h>
#include <MantleAPI/EnvironmentalConditions/weather.h>
#include <MantleAPI/Map/i_coord_converter.h>
#include <MantleAPI/Map/i_lane_location_query_service.h>
#include <MantleAPI/Traffic/i_entity_repository.h>

#include <string>

namespace mantle_api
{

class IEnvironment
{
  public:
    virtual ~IEnvironment() = default;

    virtual void Init() = 0;
    virtual void Step() = 0;

    /// Load a map file and parse it into the memory.
    ///
    /// @param file_path raw map file path from the scenario file. Resolving to a valid file path is done in the
    ///                  environment.
    virtual void CreateMap(const std::string& file_path, const std::vector<Position>& map_region) = 0;

    /// Creates a controller from the given config. A created controller can be assigned to multiple entities.
    ///
    /// @param config Specifies what kind of controller shall be created. The config needs to contain an ID to
    ///               in order to be able to assign this controller to an entity.
    virtual void CreateController(std::unique_ptr<IControllerConfig> config) = 0;

    /// Assigns an entity to a copy of the specified controller. This controller needs to be created beforehand.
    ///
    /// @param entity The entity to be manipulated by the specified controller.
    /// @param controller_id Identifies the controller to manipulate the entity.
    virtual void AddEntityToController(IEntity& entity, std::uint64_t controller_id) = 0;

    virtual const ILaneLocationQueryService& GetQueryService() const = 0;
    virtual const ICoordConverter* GetConverter() const = 0;

    virtual IEntityRepository& GetEntityRepository() = 0;
    virtual const IEntityRepository& GetEntityRepository() const = 0;

    /// @brief DateTime in UTC (converted from RFC 3339 standard)
    virtual void SetDateTime(std::chrono::duration<std::int64_t, std::milli> date_time) = 0;
    virtual std::chrono::duration<std::int64_t, std::milli> GetDateTime() = 0;

    virtual void SetWeather(Weather weather) = 0;
    virtual void SetRoadCondition(std::vector<FrictionPatch> friction_patches) = 0;
};
}  // namespace mantle_api

#endif  // MANTLEAPI_EXECUTION_IENVIRONMENT_H
