/*******************************************************************************
* Copyright (c) 2021, Bayerische Motoren Werke Aktiengesellschaft (BMW AG)
*
* This program and the accompanying materials are made
* available under the terms of the Eclipse Public License 2.0
* which is available at https://www.eclipse.org/legal/epl-2.0/
*
* SPDX-License-Identifier: EPL-2.0
*******************************************************************************/

//-----------------------------------------------------------------------------
/** @file  entity_properties.h */
//-----------------------------------------------------------------------------


#ifndef MANTLEAPI_TRAFFIC_ENTITYPROPERTIES_H
#define MANTLEAPI_TRAFFIC_ENTITYPROPERTIES_H

#include <MantleAPI/Common/dimension.h>
#include <MantleAPI/Common/floating_point_helper.h>
#include <MantleAPI/Common/vector.h>

#include <string>

namespace mantle_api
{
enum class EntityType
{
    // Other (unspecified but known) type of moving object.
    kOther = 1,
    // Object is a vehicle.
    kVehicle = 2,
    // Object is a pedestrian.
    kPedestrian = 3,
    // Object is an animal.
    kAnimal = 4
};

/// Basic properties that describe scenario entities.
struct EntityProperties
{
    virtual ~EntityProperties() = default;

    Dimension3d dimension{0.0, 0.0, 0.0};
    EntityType type{EntityType::kOther};
    std::string model{};
};

inline bool operator==(const EntityProperties& lhs, const EntityProperties& rhs) noexcept
{
    return lhs.dimension == rhs.dimension && lhs.type == rhs.type && lhs.model == rhs.model;
}

enum class VehicleClass
{
    kOther = 1,  // Other (unspecified but known) type of vehicle.
    // Vehicle is a small car.
    // Definition: Hatchback car with maximum length 4 m.
    kSmall_car = 2,
    // Vehicle is a compact car.
    // Definition: Hatchback car with length between 4 and 4.5 m.
    kCompact_car = 3,
    // Vehicle is a medium car.
    // Definition: Hatchback or sedan with length between 4.5 and 5 m.
    kMedium_car = 4,
    // Vehicle is a luxury car.
    // Definition: Sedan or coupe that is longer then 5 m.
    kLuxury_car = 5,
    // Vehicle is a delivery van.
    // Definition: A delivery van.
    kDelivery_van = 6,
    // Vehicle is a heavy truck.
    kHeavy_truck = 7,
    // Vehicle is a truck with semitrailer.
    kSemitrailer = 8,
    // Vehicle is a trailer (possibly attached to another vehicle).
    kTrailer = 9,
    // Vehicle is a motorbike or moped.
    kMotorbike = 10,
    // Vehicle is a bicycle (without motor and specific lights).
    kBicycle = 11,
    // Vehicle is a bus.
    kBus = 12,
    // Vehicle is a tram.
    kTram = 13,
    // Vehicle is a train.
    kTrain = 14,
    // Vehicle is a wheelchair.
    kWheelchair = 15,
    // Vehicle type not specified properly.
    kInvalid = -1
};

enum class IndicatorState
{
    // Indicator state is unknown (must not be used in ground truth).
    kUnknown = 0,
    // Other (unspecified but known) state of indicator.
    kOther = 1,
    // Indicators are off.
    kOff = 2,
    // Left indicator is on.
    kLeft = 3,
    // Right indicator is on.
    kRight = 4,
    // Hazard/warning light, i.e. both indicators, are on.
    kWarning = 5
};

struct VehicleProperties : public EntityProperties
{
    VehicleClass classification{VehicleClass::kOther};
    Vec3d bb_center_to_front{};
    Vec3d bb_center_to_rear{};
    double front_wheel_diameter{0.0};
    double rear_wheel_diameter{0.0};
    bool is_host{false};
};

inline bool operator==(const VehicleProperties& lhs, const VehicleProperties& rhs) noexcept
{
    return lhs.dimension == rhs.dimension && lhs.type == rhs.type && lhs.model == rhs.model &&
           lhs.classification == rhs.classification && lhs.bb_center_to_front == rhs.bb_center_to_front &&
           lhs.bb_center_to_rear == rhs.bb_center_to_rear &&
           IsEqual(lhs.front_wheel_diameter, rhs.front_wheel_diameter) &&
           IsEqual(lhs.rear_wheel_diameter, rhs.rear_wheel_diameter) && lhs.is_host == rhs.is_host;
}

struct PedestrianProperties : public EntityProperties
{
};

struct StaticObjectProperties : public EntityProperties
{
};

}  // namespace mantle_api

#endif  // MANTLEAPI_TRAFFIC_ENTITYPROPERTIES_H
